@@ -22,6 +22,7 @@ import logging
 import os
 
 from leap.bitmask import provider
+from leap.bitmask.config import flags
 from leap.bitmask.config.provider_spec import leap_provider_spec
 from leap.bitmask.services import get_service_display_name
 from leap.bitmask.util import get_path_prefix
@@ -38,62 +39,42 @@ class MissingCACert(Exception):
     pass
 
 
-class ProviderConfigLight(object):
-    """
-    A light config object to hold some provider settings needed by the GUI.
-    """
-    def __init__(self):
-        """
-        Define the public attributes.
-        """
-        self.domain = ""
-        self.name = ""
-        self.description = ""
-        self.enrollment_policy = ""
-        self.services = []
-
-    @property
-    def services_string(self):
-        """
-        Return a comma separated list of serices provided by this provider.
-
-        :rtype: str
-        """
-        services = []
-        for service in self.services:
-            services.append(get_service_display_name(service))
-
-        services_str = ", ".join(services)
-        return services_str
-
-
 class ProviderConfig(BaseConfig):
     """
     Provider configuration abstraction class
     """
     def __init__(self):
+        self.standalone = flags.STANDALONE
         BaseConfig.__init__(self)
 
     def get_light_config(self, domain, lang=None):
         """
-        Return a ProviderConfigLight object with the data for the loaded
-        object.
+        Return a dict with the data for the loaded object.
 
         :param domain: the domain name of the provider.
         :type domain: str
         :param lang: the language to use for localized strings.
         :type lang: str
 
-        :rtype: ProviderConfigLight or None if the ProviderConfig isn't loaded.
+        :rtype: dict or None if the ProviderConfig isn't loaded.
         """
         config = self.get_provider_config(domain)
-        details = ProviderConfigLight()
 
-        details.domain = config.get_domain()
-        details.name = config.get_name(lang=lang)
-        details.description = config.get_description(lang=lang)
-        details.enrollment_policy = config.get_enrollment_policy()
-        details.services = config.get_services()
+        if config is None:
+            return
+
+        details = {}
+        details["domain"] = config.get_domain()
+        details["name"] = config.get_name(lang=lang)
+        details["description"] = config.get_description(lang=lang)
+        details["enrollment_policy"] = config.get_enrollment_policy()
+        details["services"] = config.get_services()
+
+        services = []
+        for service in config.get_services():
+            services.append(get_service_display_name(service))
+
+        details['services_string'] = ", ".join(services)
 
         return details
 
@@ -220,7 +201,8 @@ class ProviderConfig(BaseConfig):
             leap_check(cert_exists, error_msg, MissingCACert)
             logger.debug("Going to verify SSL against %s" % (cert_path,))
 
-        return cert_path
+        # OpenSSL does not handle unicode.
+        return cert_path.encode('utf-8')
 
     def provides_eip(self):
         """
