@@ -17,20 +17,22 @@
 
 import binascii
 import logging
+import threading
 import sys
 
 import requests
 import srp
 import json
 
-#this error is raised from requests
+# this error is raised from requests
 from simplejson.decoder import JSONDecodeError
 from functools import partial
 from requests.adapters import HTTPAdapter
 
-from PySide import QtCore
 from twisted.internet import threads
+from twisted.internet.defer import CancelledError
 
+from leap.bitmask.backend.settings import Settings
 from leap.bitmask.util import request_helpers as reqhelper
 from leap.bitmask.util.compat import requests_has_max_retries
 from leap.bitmask.util.constants import REQUEST_TIMEOUT
@@ -116,12 +118,12 @@ class SRPAuthNoSessionId(SRPAuthenticationError):
     pass
 
 
-class SRPAuth(QtCore.QObject):
+class SRPAuth(object):
     """
     SRPAuth singleton
     """
 
-    class __impl(QtCore.QObject):
+    class __impl(object):
         """
         Implementation of the SRPAuth interface
         """
@@ -134,19 +136,22 @@ class SRPAuth(QtCore.QObject):
         USER_SALT_KEY = 'user[password_salt]'
         AUTHORIZATION_KEY = "Authorization"
 
-        def __init__(self, provider_config):
+        def __init__(self, provider_config, signaler=None):
             """
             Constructor for SRPAuth implementation
 
-            :param server: Server to which we will authenticate
-            :type server: str
+            :param provider_config: ProviderConfig needed to authenticate.
+            :type provider_config: ProviderConfig
+            :param signaler: Signaler object used to receive notifications
+                            from the backend
+            :type signaler: Signaler
             """
-            QtCore.QObject.__init__(self)
-
             leap_assert(provider_config,
                         "We need a provider config to authenticate")
 
             self._provider_config = provider_config
+            self._signaler = signaler
+            self._settings = Settings()
 
             # **************************************************** #
             # Dependency injection helpers, override this for more
@@ -160,18 +165,15 @@ class SRPAuth(QtCore.QObject):
             self._reset_session()
 
             self._session_id = None
-            self._session_id_lock = QtCore.QMutex()
-            self._uid = None
-            self._uid_lock = QtCore.QMutex()
+            self._session_id_lock = threading.Lock()
+            self._uuid = None
+            self._uuid_lock = threading.Lock()
             self._token = None
-            self._token_lock = QtCore.QMutex()
+            self._token_lock = threading.Lock()
 
             self._srp_user = None
             self._srp_a = None
 
-            # Error msg displayed if the username or the password is invalid
-            self._WRONG_USER_PASS = self.tr("Invalid username or password.")
-
             # User credentials stored for password changing checks
             self._username = None
             self._password = None
@@ -265,14 +267,11 @@ class SRPAuth(QtCore.QObject):
                 # Clean up A value, we don't need it anymore
                 self._srp_a = None
             except requests.exceptions.ConnectionError as e:
-                logger.error("No connection made (salt): %r" %
-                             (e,))
-                raise SRPAuthConnectionError("Could not establish a "
-                                             "connection")
+                logger.error("No connection made (salt): {0!r}".format(e))
+                raise SRPAuthConnectionError()
             except Exception as e:
                 logger.error("Unknown error: %r" % (e,))
-                raise SRPAuthenticationError("Unknown error: %r" %
-                                             (e,))
+                raise SRPAuthenticationError()
 
             content, mtime = reqhelper.get_content(init_session)
 
@@ -281,23 +280,22 @@ class SRPAuth(QtCore.QObject):
                              "Status code = %r. Content: %r" %
                              (init_session.status_code, content))
                 if init_session.status_code == 422:
-                    raise SRPAuthBadUserOrPassword(self._WRONG_USER_PASS)
+                    logger.error("Invalid username or password.")
+                    raise SRPAuthBadUserOrPassword()
 
-                raise SRPAuthBadStatusCode(self.tr("There was a problem with"
-                                                   " authentication"))
+                logger.error("There was a problem with authentication.")
+                raise SRPAuthBadStatusCode()
 
             json_content = json.loads(content)
             salt = json_content.get("salt", None)
             B = json_content.get("B", None)
 
             if salt is None:
-                logger.error("No salt parameter sent")
-                raise SRPAuthNoSalt(self.tr("The server did not send "
-                                            "the salt parameter"))
+                logger.error("The server didn't send the salt parameter.")
+                raise SRPAuthNoSalt()
             if B is None:
-                logger.error("No B parameter sent")
-                raise SRPAuthNoB(self.tr("The server did not send "
-                                         "the B parameter"))
+                logger.error("The server didn't send the B parameter.")
+                raise SRPAuthNoB()
 
             return salt, B
 
@@ -328,8 +326,7 @@ class SRPAuth(QtCore.QObject):
                 unhex_B = self._safe_unhexlify(B)
             except (TypeError, ValueError) as e:
                 logger.error("Bad data from server: %r" % (e,))
-                raise SRPAuthBadDataFromServer(
-                    self.tr("The data sent from the server had errors"))
+                raise SRPAuthBadDataFromServer()
             M = self._srp_user.process_challenge(unhex_salt, unhex_B)
 
             auth_url = "%s/%s/%s/%s" % (self._provider_config.get_api_uri(),
@@ -350,13 +347,13 @@ class SRPAuth(QtCore.QObject):
                                                 timeout=REQUEST_TIMEOUT)
             except requests.exceptions.ConnectionError as e:
                 logger.error("No connection made (HAMK): %r" % (e,))
-                raise SRPAuthConnectionError(self.tr("Could not connect to "
-                                                     "the server"))
+                raise SRPAuthConnectionError()
 
             try:
                 content, mtime = reqhelper.get_content(auth_result)
             except JSONDecodeError:
-                raise SRPAuthJSONDecodeError("Bad JSON content in auth result")
+                logger.error("Bad JSON content in auth result.")
+                raise SRPAuthJSONDecodeError()
 
             if auth_result.status_code == 422:
                 error = ""
@@ -370,14 +367,13 @@ class SRPAuth(QtCore.QObject):
                                  "received: %s", (content,))
                 logger.error("[%s] Wrong password (HAMK): [%s]" %
                              (auth_result.status_code, error))
-                raise SRPAuthBadUserOrPassword(self._WRONG_USER_PASS)
+                raise SRPAuthBadUserOrPassword()
 
             if auth_result.status_code not in (200,):
                 logger.error("No valid response (HAMK): "
                              "Status code = %s. Content = %r" %
                              (auth_result.status_code, content))
-                raise SRPAuthBadStatusCode(self.tr("Unknown error (%s)") %
-                                           (auth_result.status_code,))
+                raise SRPAuthBadStatusCode()
 
             return json.loads(content)
 
@@ -394,24 +390,22 @@ class SRPAuth(QtCore.QObject):
             """
             try:
                 M2 = json_content.get("M2", None)
-                uid = json_content.get("id", None)
+                uuid = json_content.get("id", None)
                 token = json_content.get("token", None)
             except Exception as e:
                 logger.error(e)
-                raise SRPAuthBadDataFromServer("Something went wrong with the "
-                                               "login")
+                raise SRPAuthBadDataFromServer()
 
-            self.set_uid(uid)
+            self.set_uuid(uuid)
             self.set_token(token)
 
-            if M2 is None or self.get_uid() is None:
+            if M2 is None or self.get_uuid() is None:
                 logger.error("Something went wrong. Content = %r" %
                              (json_content,))
-                raise SRPAuthBadDataFromServer(self.tr("Problem getting data "
-                                                       "from server"))
+                raise SRPAuthBadDataFromServer()
 
             events_signal(
-                proto.CLIENT_UID, content=uid,
+                proto.CLIENT_UID, content=uuid,
                 reqcbk=lambda req, res: None)  # make the rpc call async
 
             return M2
@@ -434,22 +428,19 @@ class SRPAuth(QtCore.QObject):
                 unhex_M2 = self._safe_unhexlify(M2)
             except TypeError:
                 logger.error("Bad data from server (HAWK)")
-                raise SRPAuthBadDataFromServer(self.tr("Bad data from server"))
+                raise SRPAuthBadDataFromServer()
 
             self._srp_user.verify_session(unhex_M2)
 
             if not self._srp_user.authenticated():
-                logger.error("Auth verification failed")
-                raise SRPAuthVerificationFailed(self.tr("Auth verification "
-                                                        "failed"))
+                logger.error("Auth verification failed.")
+                raise SRPAuthVerificationFailed()
             logger.debug("Session verified.")
 
             session_id = self._session.cookies.get(self.SESSION_ID_KEY, None)
             if not session_id:
                 logger.error("Bad cookie from server (missing _session_id)")
-                raise SRPAuthNoSessionId(self.tr("Session cookie "
-                                                 "verification "
-                                                 "failed"))
+                raise SRPAuthNoSessionId()
 
             events_signal(
                 proto.CLIENT_SESSION_ID, content=session_id,
@@ -460,7 +451,7 @@ class SRPAuth(QtCore.QObject):
         def _threader(self, cb, res, *args, **kwargs):
             return threads.deferToThread(cb, res, *args, **kwargs)
 
-        def change_password(self, current_password, new_password):
+        def _change_password(self, current_password, new_password):
             """
             Changes the password for the currently logged user if the current
             password match.
@@ -475,7 +466,7 @@ class SRPAuth(QtCore.QObject):
             :param new_password: the new password for the user
             :type new_password: str
             """
-            leap_assert(self.get_uid() is not None)
+            leap_assert(self.get_uuid() is not None)
 
             if current_password != self._password:
                 raise SRPAuthBadUserOrPassword
@@ -483,7 +474,7 @@ class SRPAuth(QtCore.QObject):
             url = "%s/%s/users/%s.json" % (
                 self._provider_config.get_api_uri(),
                 self._provider_config.get_api_version(),
-                self.get_uid())
+                self.get_uuid())
 
             salt, verifier = self._srp.create_salted_verification_key(
                 self._username.encode('utf-8'), new_password.encode('utf-8'),
@@ -511,6 +502,43 @@ class SRPAuth(QtCore.QObject):
 
             self._password = new_password
 
+        def change_password(self, current_password, new_password):
+            """
+            Changes the password for the currently logged user if the current
+            password match.
+            It requires to be authenticated.
+
+            :param current_password: the current password for the logged user.
+            :type current_password: str
+            :param new_password: the new password for the user
+            :type new_password: str
+            """
+            d = threads.deferToThread(
+                self._change_password, current_password, new_password)
+            d.addCallback(self._change_password_ok)
+            d.addErrback(self._change_password_error)
+
+        def _change_password_ok(self, _):
+            """
+            Password change callback.
+            """
+            if self._signaler is not None:
+                self._signaler.signal(self._signaler.srp_password_change_ok)
+
+        def _change_password_error(self, failure):
+            """
+            Password change errback.
+            """
+            logger.debug(
+                "Error changing password. Failure: {0}".format(failure))
+            if self._signaler is None:
+                return
+
+            if failure.check(SRPAuthBadUserOrPassword):
+                self._signaler.signal(self._signaler.srp_password_change_badpw)
+            else:
+                self._signaler.signal(self._signaler.srp_password_change_error)
+
         def authenticate(self, username, password):
             """
             Executes the whole authentication process for a user
@@ -551,8 +579,49 @@ class SRPAuth(QtCore.QObject):
             d.addCallback(partial(self._threader,
                                   self._verify_session))
 
+            d.addCallback(self._authenticate_ok)
+            d.addErrback(self._authenticate_error)
             return d
 
+        def _authenticate_ok(self, _):
+            """
+            Callback that notifies that the authentication was successful.
+
+            :param _: IGNORED, output from the previous callback (None)
+            :type _: IGNORED
+            """
+            logger.debug("Successful login!")
+            self._signaler.signal(self._signaler.srp_auth_ok)
+
+        def _authenticate_error(self, failure):
+            """
+            Error handler for the srpauth.authenticate method.
+
+            :param failure: failure object that Twisted generates
+            :type failure: twisted.python.failure.Failure
+            """
+            logger.error("Error logging in, {0!r}".format(failure))
+
+            signal = None
+            if failure.check(CancelledError):
+                logger.debug("Defer cancelled.")
+                failure.trap(Exception)
+                return
+
+            if self._signaler is None:
+                return
+
+            if failure.check(SRPAuthBadUserOrPassword):
+                signal = self._signaler.srp_auth_bad_user_or_password
+            elif failure.check(SRPAuthConnectionError):
+                signal = self._signaler.srp_auth_connection_error
+            elif failure.check(SRPAuthenticationError):
+                signal = self._signaler.srp_auth_server_error
+            else:
+                signal = self._signaler.srp_auth_error
+
+            self._signaler.signal(signal)
+
         def logout(self):
             """
             Logs out the current session.
@@ -577,53 +646,75 @@ class SRPAuth(QtCore.QObject):
             except Exception as e:
                 logger.warning("Something went wrong with the logout: %r" %
                                (e,))
+                if self._signaler is not None:
+                    self._signaler.signal(self._signaler.srp_logout_error)
                 raise
             else:
                 self.set_session_id(None)
-                self.set_uid(None)
+                self.set_uuid(None)
+                self.set_token(None)
                 # Also reset the session
                 self._session = self._fetcher.session()
                 logger.debug("Successfully logged out.")
+                if self._signaler is not None:
+                    self._signaler.signal(self._signaler.srp_logout_ok)
 
         def set_session_id(self, session_id):
-            QtCore.QMutexLocker(self._session_id_lock)
-            self._session_id = session_id
+            with self._session_id_lock:
+                self._session_id = session_id
 
         def get_session_id(self):
-            QtCore.QMutexLocker(self._session_id_lock)
-            return self._session_id
+            with self._session_id_lock:
+                return self._session_id
 
-        def set_uid(self, uid):
-            QtCore.QMutexLocker(self._uid_lock)
-            self._uid = uid
+        def set_uuid(self, uuid):
+            with self._uuid_lock:
+                full_uid = "%s@%s" % (
+                    self._username, self._provider_config.get_domain())
+                if uuid is not None:  # avoid removing the uuid from settings
+                    self._settings.set_uuid(full_uid, uuid)
+                self._uuid = uuid
 
-        def get_uid(self):
-            QtCore.QMutexLocker(self._uid_lock)
-            return self._uid
+        def get_uuid(self):
+            with self._uuid_lock:
+                return self._uuid
 
         def set_token(self, token):
-            QtCore.QMutexLocker(self._token_lock)
-            self._token = token
+            with self._token_lock:
+                self._token = token
 
         def get_token(self):
-            QtCore.QMutexLocker(self._token_lock)
-            return self._token
+            with self._token_lock:
+                return self._token
+
+        def is_authenticated(self):
+            """
+            Return whether the user is authenticated or not.
+
+            :rtype: bool
+            """
+            user = self._srp_user
+            if user is not None:
+                return user.authenticated()
+
+            return False
 
     __instance = None
 
-    authentication_finished = QtCore.Signal(bool, str)
-    logout_finished = QtCore.Signal(bool, str)
-
-    def __init__(self, provider_config):
+    def __init__(self, provider_config, signaler=None):
         """
-        Creates a singleton instance if needed
-        """
-        QtCore.QObject.__init__(self)
+        Create a singleton instance if needed
 
+        :param provider_config: ProviderConfig needed to authenticate.
+        :type provider_config: ProviderConfig
+        :param signaler: Signaler object used to send notifications
+                         from the backend
+        :type signaler: Signaler
+        """
         # Check whether we already have an instance
         if SRPAuth.__instance is None:
             # Create and remember instance
-            SRPAuth.__instance = SRPAuth.__impl(provider_config)
+            SRPAuth.__instance = SRPAuth.__impl(provider_config, signaler)
 
         # Store instance reference as the only member in the handle
         self.__dict__['_SRPAuth__instance'] = SRPAuth.__instance
@@ -648,10 +739,16 @@ class SRPAuth(QtCore.QObject):
         """
         username = username.lower()
         d = self.__instance.authenticate(username, password)
-        d.addCallback(self._gui_notify)
-        d.addErrback(self._errback)
         return d
 
+    def is_authenticated(self):
+        """
+        Return whether the user is authenticated or not.
+
+        :rtype: bool
+        """
+        return self.__instance.is_authenticated()
+
     def change_password(self, current_password, new_password):
         """
         Changes the user's password.
@@ -664,8 +761,7 @@ class SRPAuth(QtCore.QObject):
         :returns: a defer to interact with.
         :rtype: twisted.internet.defer.Deferred
         """
-        d = threads.deferToThread(
-            self.__instance.change_password, current_password, new_password)
+        d = self.__instance.change_password(current_password, new_password)
         return d
 
     def get_username(self):
@@ -675,37 +771,15 @@ class SRPAuth(QtCore.QObject):
 
         :rtype: str or None
         """
-        if self.get_uid() is None:
+        if self.get_uuid() is None:
             return None
         return self.__instance._username
 
-    def _gui_notify(self, _):
-        """
-        Callback that notifies the UI with the proper signal.
-
-        :param _: IGNORED, output from the previous callback (None)
-        :type _: IGNORED
-        """
-        logger.debug("Successful login!")
-        self.authentication_finished.emit(True, self.tr("Succeeded"))
-
-    def _errback(self, failure):
-        """
-        General errback for the whole login process. Will notify the
-        UI with the proper signal.
-
-        :param failure: Failure object captured from a callback.
-        :type failure: twisted.python.failure.Failure
-        """
-        logger.error("Error logging in %s" % (failure,))
-        self.authentication_finished.emit(False, "%s" % (failure.value,))
-        failure.trap(Exception)
-
     def get_session_id(self):
         return self.__instance.get_session_id()
 
-    def get_uid(self):
-        return self.__instance.get_uid()
+    def get_uuid(self):
+        return self.__instance.get_uuid()
 
     def get_token(self):
         return self.__instance.get_token()
@@ -717,8 +791,8 @@ class SRPAuth(QtCore.QObject):
         """
         try:
             self.__instance.logout()
-            self.logout_finished.emit(True, self.tr("Succeeded"))
+            logger.debug("Logout success")
             return True
         except Exception as e:
-            self.logout_finished.emit(False, "%s" % (e,))
+            logger.debug("Logout error: {0!r}".format(e))
         return False
