@@ -19,12 +19,14 @@ Login widget implementation
 """
 import logging
 
-import keyring
-
 from PySide import QtCore, QtGui
 from ui_login import Ui_LoginWidget
 
+from leap.bitmask.config import flags
+from leap.bitmask.util import make_address
+from leap.bitmask.util.credentials import USERNAME_REGEX
 from leap.bitmask.util.keyring_helpers import has_keyring
+from leap.bitmask.util.keyring_helpers import get_keyring
 from leap.common.check import leap_assert_type
 
 logger = logging.getLogger(__name__)
@@ -41,13 +43,10 @@ class LoginWidget(QtGui.QWidget):
     cancel_login = QtCore.Signal()
     logout = QtCore.Signal()
 
-    # Emitted when the user selects "Other..." in the provider
-    # combobox or click "Create Account"
-    show_wizard = QtCore.Signal()
-
     MAX_STATUS_WIDTH = 40
 
-    BARE_USERNAME_REGEX = r"^[A-Za-z\d_]+$"
+    # Keyring
+    KEYRING_KEY = "bitmask"
 
     def __init__(self, settings, parent=None):
         """
@@ -61,7 +60,6 @@ class LoginWidget(QtGui.QWidget):
         QtGui.QWidget.__init__(self, parent)
 
         self._settings = settings
-        self._selected_provider_index = -1
 
         self.ui = Ui_LoginWidget()
         self.ui.setupUi(self)
@@ -77,13 +75,10 @@ class LoginWidget(QtGui.QWidget):
 
         self.ui.lnUser.returnPressed.connect(self._focus_password)
 
-        self.ui.cmbProviders.currentIndexChanged.connect(
-            self._current_provider_changed)
-
         self.ui.btnLogout.clicked.connect(
             self.logout)
 
-        username_re = QtCore.QRegExp(self.BARE_USERNAME_REGEX)
+        username_re = QtCore.QRegExp(USERNAME_REGEX)
         self.ui.lnUser.setValidator(
             QtGui.QRegExpValidator(username_re, self))
 
@@ -94,6 +89,9 @@ class LoginWidget(QtGui.QWidget):
         self.ui.clblErrorMsg.hide()
         self.ui.clblErrorMsg.clicked.connect(self.ui.clblErrorMsg.hide)
 
+        self.ui.lnUser.textEdited.connect(self.ui.clblErrorMsg.hide)
+        self.ui.lnPassword.textEdited.connect(self.ui.clblErrorMsg.hide)
+
     def _remember_state_changed(self, state):
         """
         Saves the remember state in the LeapSettings
@@ -105,35 +103,6 @@ class LoginWidget(QtGui.QWidget):
         enable = True if state == QtCore.Qt.Checked else False
         self._settings.set_remember(enable)
 
-    def set_providers(self, provider_list):
-        """
-        Set the provider list to provider_list plus an "Other..." item
-        that triggers the wizard
-
-        :param provider_list: list of providers
-        :type provider_list: list of str
-        """
-        self.ui.cmbProviders.blockSignals(True)
-        self.ui.cmbProviders.clear()
-        self.ui.cmbProviders.addItems(provider_list + [self.tr("Other...")])
-        self.ui.cmbProviders.blockSignals(False)
-
-    def select_provider_by_name(self, name):
-        """
-        Given a provider name/domain, it selects it in the combobox
-
-        :param name: name or domain for the provider
-        :type name: str
-        """
-        provider_index = self.ui.cmbProviders.findText(name)
-        self.ui.cmbProviders.setCurrentIndex(provider_index)
-
-    def get_selected_provider(self):
-        """
-        Returns the selected provider in the combobox
-        """
-        return self.ui.cmbProviders.currentText()
-
     def set_remember(self, value):
         """
         Checks the remember user and password checkbox
@@ -165,7 +134,7 @@ class LoginWidget(QtGui.QWidget):
         """
         Returns the user that appears in the widget.
 
-        :rtype: str
+        :rtype: unicode
         """
         return self.ui.lnUser.text()
 
@@ -174,7 +143,7 @@ class LoginWidget(QtGui.QWidget):
         Sets the password for the widget
 
         :param password: password to set
-        :type password: str
+        :type password: unicode
         """
         self.ui.lnPassword.setText(password)
 
@@ -182,7 +151,7 @@ class LoginWidget(QtGui.QWidget):
         """
         Returns the password that appears in the widget
 
-        :rtype: str
+        :rtype: unicode
         """
         return self.ui.lnPassword.text()
 
@@ -210,11 +179,19 @@ class LoginWidget(QtGui.QWidget):
         """
         self.ui.lnUser.setEnabled(enabled)
         self.ui.lnPassword.setEnabled(enabled)
-        self.ui.chkRemember.setEnabled(enabled)
-        self.ui.cmbProviders.setEnabled(enabled)
+        self.ui.chkRemember.setEnabled(enabled and has_keyring())
 
         self._set_cancel(not enabled)
 
+    def set_logout_btn_enabled(self, enabled):
+        """
+        Enables or disables the logout button.
+
+        :param enabled: wether they should be enabled or not
+        :type enabled: bool
+        """
+        self.ui.btnLogout.setEnabled(enabled)
+
     def _set_cancel(self, enabled=False):
         """
         Enables or disables the cancel action in the "log in" process.
@@ -243,37 +220,20 @@ class LoginWidget(QtGui.QWidget):
         """
         self.ui.lnPassword.setFocus()
 
-    def _current_provider_changed(self, param):
-        """
-        SLOT
-        TRIGGERS: self.ui.cmbProviders.currentIndexChanged
-        """
-        if param == (self.ui.cmbProviders.count() - 1):
-            self.show_wizard.emit()
-            # Leave the previously selected provider in the combobox
-            prev_provider = 0
-            if self._selected_provider_index != -1:
-                prev_provider = self._selected_provider_index
-            self.ui.cmbProviders.blockSignals(True)
-            self.ui.cmbProviders.setCurrentIndex(prev_provider)
-            self.ui.cmbProviders.blockSignals(False)
-        else:
-            self._selected_provider_index = param
-
-    def start_login(self):
+    def start_login(self, provider):
         """
         Setups the login widgets for actually performing the login and
         performs some basic checks.
 
+        :param provider: the domain of the current provider
+        :type provider: unicode str
         :returns: True if everything's good to go, False otherwise
         :rtype: bool
         """
         username = self.get_user()
         password = self.get_password()
-        provider = self.get_selected_provider()
 
-        self._enabled_services = self._settings.get_enabled_services(
-            self.get_selected_provider())
+        self._enabled_services = self._settings.get_enabled_services(provider)
 
         if len(provider) == 0:
             self.set_status(
@@ -294,37 +254,45 @@ class LoginWidget(QtGui.QWidget):
         self.set_enabled(False)
         self.ui.clblErrorMsg.hide()
 
+        self._settings.set_provider(provider)
         if self.get_remember() and has_keyring():
             # in the keyring and in the settings
             # we store the value 'usename@provider'
-            username_domain = (username + '@' + provider).encode("utf8")
+            full_user_id = make_address(username, provider).encode("utf8")
             try:
+                keyring = get_keyring()
                 keyring.set_password(self.KEYRING_KEY,
-                                     username_domain,
+                                     full_user_id,
                                      password.encode("utf8"))
                 # Only save the username if it was saved correctly in
                 # the keyring
-                self._settings.set_user(username_domain)
+                self._settings.set_user(full_user_id)
             except Exception as e:
                 logger.exception("Problem saving data to keyring. %r"
                                  % (e,))
         return True
 
-    def logged_in(self):
+    def logged_in(self, provider):
         """
         Sets the widgets to the logged in state
+
+        :param provider: the domain of the current provider
+        :type provider: unicode str
         """
         self.ui.login_widget.hide()
         self.ui.logged_widget.show()
-        self.ui.lblUser.setText("%s@%s" % (self.get_user(),
-                                           self.get_selected_provider()))
-        self.set_login_status("")
-        self.logged_in_signal.emit()
+        self.ui.lblUser.setText(make_address(self.get_user(), provider))
+
+        if flags.OFFLINE is False:
+            self.logged_in_signal.emit()
 
     def logged_out(self):
         """
         Sets the widgets to the logged out state
         """
+        # TODO consider "logging out offline" too...
+        # how that would be ???
+
         self.ui.login_widget.show()
         self.ui.logged_widget.hide()
 
@@ -332,27 +300,11 @@ class LoginWidget(QtGui.QWidget):
         self.set_enabled(True)
         self.set_status("", error=False)
 
-    def set_login_status(self, msg, error=False):
-        """
-        Sets the status label for the logged in state.
-
-        :param msg: status message
-        :type msg: str or unicode
-        :param error: if the status is an erroneous one, then set this
-                      to True
-        :type error: bool
-        """
-        leap_assert_type(error, bool)
-        if error:
-            msg = "<font color='red'><b>%s</b></font>" % (msg,)
-        self.ui.lblLoginStatus.setText(msg)
-        self.ui.lblLoginStatus.show()
-
     def start_logout(self):
         """
         Sets the widgets to the logging out state
         """
-        self.ui.btnLogout.setText(self.tr("Loggin out..."))
+        self.ui.btnLogout.setText(self.tr("Logging out..."))
         self.ui.btnLogout.setEnabled(False)
 
     def done_logout(self):
@@ -362,3 +314,42 @@ class LoginWidget(QtGui.QWidget):
         self.ui.btnLogout.setText(self.tr("Logout"))
         self.ui.btnLogout.setEnabled(True)
         self.ui.clblErrorMsg.hide()
+
+    def load_user_from_keyring(self, saved_user):
+        """
+        Tries to load a user from the keyring, returns True if it was
+        loaded successfully, False otherwise.
+
+        :param saved_user: String containing the saved username as
+                           user@domain
+        :type saved_user: unicode
+
+        :rtype: bool
+        """
+        leap_assert_type(saved_user, unicode)
+
+        try:
+            username, domain = saved_user.split('@')
+        except ValueError as e:
+            # if the saved_user does not contain an '@'
+            logger.error('Username@provider malformed. %r' % (e, ))
+            return False
+
+        self.set_user(username)
+
+        self.set_remember(True)
+
+        saved_password = None
+        try:
+            keyring = get_keyring()
+            saved_password = keyring.get_password(self.KEYRING_KEY,
+                                                  saved_user
+                                                  .encode("utf8"))
+        except ValueError as e:
+            logger.debug("Incorrect Password. %r." % (e,))
+
+        if saved_password is not None:
+            self.set_password(saved_password)
+            return True
+
+        return False
