@@ -240,9 +240,9 @@ class CompositeMachine(QStateMachine):
         c2.qtsigs.disconnected_signal.connect(self.off_ev2_slot)
 
     # XXX why is this getting deletec in c++?
-    #Traceback (most recent call last):
-    #self.postEvent(self.events.on_ev2)
-    #RuntimeError: Internal C++ object (ConnectedEvent2) already deleted.
+    # Traceback (most recent call last):
+    # self.postEvent(self.events.on_ev2)
+    # RuntimeError: Internal C++ object (ConnectedEvent2) already deleted.
     # XXX trying the following workaround, since
     # I cannot find why in the world this is getting deleted :(
     # XXX refactor!
@@ -318,9 +318,8 @@ class ConnectionMachineBuilder(object):
         components = self._conn.components
 
         if components is None:
-        # simple case: connection definition inherits directly from
-        # the abstract connection.
-
+            # simple case: connection definition inherits directly from
+            # the abstract connection.
             leap_assert_type(self._conn, connections.AbstractLEAPConnection)
             return self._make_simple_machine(self._conn, **kwargs)
 
@@ -357,7 +356,7 @@ class ConnectionMachineBuilder(object):
         parent = kwargs.get('parent', None)
 
         # 1. create machine
-        machine = CompositeMachine(parent=parent)
+        machine = CompositeMachine()
 
         # 2. create states
         off = States.Off(conn.qtsigs.disconnected_signal,
@@ -504,6 +503,11 @@ class ConnectionMachineBuilder(object):
             conn.qtsigs.connection_died_signal,
             states[_OFF])
 
+        # XXX adding this---------------------
+        states[_ON].addTransition(
+            conn.qtsigs.do_disconnect_signal,
+            states[_DIS])
+
         # * If we receive the connection_aborted, we transition
         #   from connecting to the off state
         states[_CON].addTransition(
@@ -551,7 +555,8 @@ class ConnectionMachineBuilder(object):
         # TODO add tooltip
 
         # OFF State ----------------------
-        off = QState()
+        off = SignallingState(
+            None, name=conn.name)
         off_label = _tr("Turn {0}").format(
             conn.Connected.short_label)
         if button:
@@ -559,9 +564,15 @@ class ConnectionMachineBuilder(object):
                 button, 'text', off_label)
             off.assignProperty(
                 button, 'enabled', True)
+            off.assignProperty(
+                button, 'visible', True)
         if action:
             off.assignProperty(
                 action, 'text', off_label)
+            off.assignProperty(
+                action, 'enabled', True)
+            off.assignProperty(
+                action, 'visible', True)
         off.setObjectName(_OFF)
         states[_OFF] = off
 
@@ -585,7 +596,10 @@ class ConnectionMachineBuilder(object):
         states[_CON] = connecting
 
         # ON State ------------------------
-        on = QState()
+        on = SignallingState(
+            None, name=conn.name)
+        on_label = _tr("Turn {0}").format(
+            conn.Disconnected.short_label)
         if button:
             on.assignProperty(
                 button, 'text', on_label)
