@@ -24,6 +24,10 @@ from functools import partial
 
 from PySide import QtCore, QtGui
 
+# TODO: we should use a more granular signaling instead of passing error/ok as
+# a result.
+from leap.bitmask.backend.leapbackend import ERROR_KEY, PASSED_KEY
+
 from leap.bitmask.config import flags
 from leap.bitmask.config.leapsettings import LeapSettings
 from leap.bitmask.services import get_service_display_name, get_supported
@@ -33,10 +37,12 @@ from leap.bitmask.util.keyring_helpers import has_keyring
 
 from ui_wizard import Ui_Wizard
 
+QtDelayedCall = QtCore.QTimer.singleShot
 logger = logging.getLogger(__name__)
 
 
 class Wizard(QtGui.QWizard):
+
     """
     First run wizard to register a user and setup a provider
     """
@@ -48,28 +54,26 @@ class Wizard(QtGui.QWizard):
     REGISTER_USER_PAGE = 4
     SERVICES_PAGE = 5
 
-    def __init__(self, backend, bypass_checks=False):
+    def __init__(self, backend, leap_signaler):
         """
         Constructor for the main Wizard.
 
         :param backend: Backend being used
         :type backend: Backend
-        :param bypass_checks: Set to true if the app should bypass
-                              first round of checks for CA
-                              certificates at bootstrap
-        :type bypass_checks: bool
         """
         QtGui.QWizard.__init__(self)
 
         self.ui = Ui_Wizard()
         self.ui.setupUi(self)
 
+        self._connected_signals = []
+
         self.setPixmap(QtGui.QWizard.LogoPixmap,
                        QtGui.QPixmap(":/images/mask-icon.png"))
 
-        self.QUESTION_ICON = QtGui.QPixmap(":/images/Emblem-question.png")
-        self.ERROR_ICON = QtGui.QPixmap(":/images/Dialog-error.png")
-        self.OK_ICON = QtGui.QPixmap(":/images/Dialog-accept.png")
+        self.QUESTION_ICON = QtGui.QPixmap(":/images/black/22/question.png")
+        self.ERROR_ICON = QtGui.QPixmap(":/images/black/22/off.png")
+        self.OK_ICON = QtGui.QPixmap(":/images/black/22/on.png")
 
         self._selected_services = set()
         self._shown_services = set()
@@ -79,8 +83,11 @@ class Wizard(QtGui.QWizard):
         self._use_existing_provider = False
 
         self.ui.grpCheckProvider.setVisible(False)
-        self.ui.btnCheck.clicked.connect(self._check_provider)
-        self.ui.lnProvider.returnPressed.connect(self._check_provider)
+        self._connect_and_track(self.ui.btnCheck.clicked, self._check_provider)
+        self._connect_and_track(self.ui.lnProvider.returnPressed,
+                                self._check_provider)
+
+        self._leap_signaler = leap_signaler
 
         self._backend = backend
         self._backend_connect()
@@ -90,29 +97,27 @@ class Wizard(QtGui.QWizard):
         # this details are set when the provider download is complete.
         self._provider_details = None
 
-        # We will store a reference to the defers for eventual use
-        # (eg, to cancel them) but not doing anything with them right now.
-        self._provider_select_defer = None
-        self._provider_setup_defer = None
+        self._connect_and_track(self.currentIdChanged,
+                                self._current_id_changed)
 
-        self.currentIdChanged.connect(self._current_id_changed)
+        self._connect_and_track(self.ui.lnProvider.textChanged,
+                                self._enable_check)
+        self._connect_and_track(self.ui.rbNewProvider.toggled,
+                                lambda x: self._enable_check())
+        self._connect_and_track(self.ui.cbProviders.currentIndexChanged[int],
+                                self._reset_provider_check)
 
-        self.ui.lnProvider.textChanged.connect(self._enable_check)
-        self.ui.rbNewProvider.toggled.connect(
-            lambda x: self._enable_check())
-        self.ui.cbProviders.currentIndexChanged[int].connect(
-            self._reset_provider_check)
+        self._connect_and_track(self.ui.lblUser.returnPressed,
+                                self._focus_password)
+        self._connect_and_track(self.ui.lblPassword.returnPressed,
+                                self._focus_second_password)
+        self._connect_and_track(self.ui.lblPassword2.returnPressed,
+                                self._register)
+        self._connect_and_track(self.ui.btnRegister.clicked,
+                                self._register)
 
-        self.ui.lblUser.returnPressed.connect(
-            self._focus_password)
-        self.ui.lblPassword.returnPressed.connect(
-            self._focus_second_password)
-        self.ui.lblPassword2.returnPressed.connect(
-            self._register)
-        self.ui.btnRegister.clicked.connect(
-            self._register)
-
-        self.ui.rbExistingProvider.toggled.connect(self._skip_provider_checks)
+        self._connect_and_track(self.ui.rbExistingProvider.toggled,
+                                self._skip_provider_checks)
 
         usernameRe = QtCore.QRegExp(USERNAME_REGEX)
         self.ui.lblUser.setValidator(
@@ -137,7 +142,19 @@ class Wizard(QtGui.QWizard):
 
         self._provider_checks_ok = False
         self._provider_setup_ok = False
-        self.finished.connect(self._wizard_finished)
+        self._connect_and_track(self.finished, self._wizard_finished)
+
+    def _connect_and_track(self, signal, method):
+        """
+        Helper to connect signals and keep track of them.
+
+        :param signal: the signal to connect to.
+        :type signal: QtCore.Signal
+        :param method: the method to call when the signal is triggered.
+        :type method: callable, Slot or Signal
+        """
+        self._connected_signals.append((signal, method))
+        signal.connect(method)
 
     @QtCore.Slot()
     def _wizard_finished(self):
@@ -153,29 +170,70 @@ class Wizard(QtGui.QWizard):
         self._provider_setup_ok = False
         self.ui.lnProvider.setText('')
         self.ui.grpCheckProvider.setVisible(False)
-        self._backend_disconnect()
+        self._disconnect_tracked()
 
     def _load_configured_providers(self):
         """
         Loads the configured providers into the wizard providers combo box.
         """
+        self._backend.provider_get_pinned_providers()
+
+    def _load_configured_providers_with_pinned(self, pinned):
+        """
+        Once we have the pinned providers from the backend, we
+        continue setting everything up
+
+        :param pinned: list of pinned providers
+        :type pinned: list of str
+
+
+        How the combobox items are arranged:
+        -----------------------------------
+
+        First run:
+
+            demo.bitmask.net
+            --
+            pinned2.org
+            pinned1.org
+            pinned3.org
+
+        After some usage:
+
+            added-by-user.org
+            pinned-but-then-used.org
+            ---
+            demo.bitmask.net
+            pinned1.org
+            pinned3.org
+            pinned2.org
+
+        In other words:
+            * There are two sections.
+            * Section one consists of all the providers that the user has used.
+              If this is empty, than use demo.bitmask.net for this section.
+              This list is sorted alphabetically.
+            * Section two consists of all the pinned or 'pre seeded' providers,
+              minus any providers that are now in section one. This last list
+              is in random order.
+        """
         ls = LeapSettings()
-        providers = ls.get_configured_providers()
-        if not providers:
+        user_added = ls.get_configured_providers()
+        if not user_added and not pinned:
             self.ui.rbExistingProvider.setEnabled(False)
             self.ui.label_8.setEnabled(False)  # 'https://' label
             self.ui.cbProviders.setEnabled(False)
             return
 
-        pinned = []
-        user_added = []
+        user_added.sort()
 
-        # separate pinned providers from user added ones
-        for p in providers:
-            if ls.is_pinned_provider(p):
-                pinned.append(p)
-            else:
-                user_added.append(p)
+        if not user_added:
+            user_added = [pinned.pop(0)]
+
+        # separate unused pinned providers from user added ones
+        for p in user_added:
+            if p in pinned:
+                pinned.remove(p)
 
         if user_added:
             self.ui.cbProviders.addItems(user_added)
@@ -191,6 +249,9 @@ class Wizard(QtGui.QWizard):
         # 'Use existing provider' option.
         self.ui.rbExistingProvider.setChecked(True)
 
+        # We need to set it as complete explicitly
+        self.page(self.INTRO_PAGE).set_completed()
+
     def get_domain(self):
         return self._domain
 
@@ -257,12 +318,14 @@ class Wizard(QtGui.QWizard):
 
         user_ok, msg = username_checks(username)
         if user_ok:
-            pass_ok, msg = password_checks(username, password, password2)
+            pass_ok, msg, field = password_checks(
+                username, password, password2)
 
         if user_ok and pass_ok:
             self._set_register_status(self.tr("Starting registration..."))
 
-            self._backend.user_register(self._domain, username, password)
+            self._backend.user_register(provider=self._domain,
+                                        username=username, password=password)
             self._username = username
             self._password = password
         else:
@@ -413,8 +476,7 @@ class Wizard(QtGui.QWizard):
         self.button(QtGui.QWizard.BackButton).clearFocus()
 
         self.ui.lblNameResolution.setPixmap(self.QUESTION_ICON)
-        self._provider_select_defer = self._backend.\
-            provider_setup(self._domain)
+        self._backend.provider_setup(provider=self._domain)
 
     @QtCore.Slot(bool)
     def _skip_provider_checks(self, skip):
@@ -449,8 +511,8 @@ class Wizard(QtGui.QWizard):
         :param complete_page: page id to complete
         :type complete_page: int
         """
-        passed = data[self._backend.PASSED_KEY]
-        error = data[self._backend.ERROR_KEY]
+        passed = data[PASSED_KEY]
+        error = data[ERROR_KEY]
         if passed:
             label.setPixmap(self.OK_ICON)
             if complete:
@@ -470,7 +532,7 @@ class Wizard(QtGui.QWizard):
         """
         self._complete_task(data, self.ui.lblNameResolution)
         status = ""
-        passed = data[self._backend.PASSED_KEY]
+        passed = data[PASSED_KEY]
         if not passed:
             status = self.tr("<font color='red'><b>Non-existent "
                              "provider</b></font>")
@@ -490,10 +552,10 @@ class Wizard(QtGui.QWizard):
         """
         self._complete_task(data, self.ui.lblHTTPS)
         status = ""
-        passed = data[self._backend.PASSED_KEY]
+        passed = data[PASSED_KEY]
         if not passed:
             status = self.tr("<font color='red'><b>%s</b></font>") \
-                % (data[self._backend.ERROR_KEY])
+                % (data[ERROR_KEY])
             self.ui.lblProviderSelectStatus.setText(status)
         else:
             self.ui.lblProviderInfo.setPixmap(self.QUESTION_ICON)
@@ -510,22 +572,21 @@ class Wizard(QtGui.QWizard):
         check. Since this check is the last of this set, it also
         completes the page if passed
         """
-        if data[self._backend.PASSED_KEY]:
+        if data[PASSED_KEY]:
             self._complete_task(data, self.ui.lblProviderInfo,
                                 True, self.SELECT_PROVIDER_PAGE)
             self._provider_checks_ok = True
             lang = QtCore.QLocale.system().name()
-            self._backend.provider_get_details(self._domain, lang)
+            self._backend.provider_get_details(domain=self._domain, lang=lang)
         else:
             new_data = {
-                self._backend.PASSED_KEY: False,
-                self._backend.ERROR_KEY:
-                self.tr("Unable to load provider configuration")
+                PASSED_KEY: False,
+                ERROR_KEY: self.tr("Unable to load provider configuration")
             }
             self._complete_task(new_data, self.ui.lblProviderInfo)
 
         status = ""
-        if not data[self._backend.PASSED_KEY]:
+        if not data[PASSED_KEY]:
             status = self.tr("<font color='red'><b>Not a valid provider"
                              "</b></font>")
             self.ui.lblProviderSelectStatus.setText(status)
@@ -543,7 +604,7 @@ class Wizard(QtGui.QWizard):
         Set the details for the just downloaded provider.
 
         :param details: the details of the provider.
-        :type details: ProviderConfigLight
+        :type details: dict
         """
         self._provider_details = details
 
@@ -556,7 +617,7 @@ class Wizard(QtGui.QWizard):
         Sets the status for the download of the CA certificate check
         """
         self._complete_task(data, self.ui.lblDownloadCaCert)
-        passed = data[self._backend.PASSED_KEY]
+        passed = data[PASSED_KEY]
         if passed:
             self.ui.lblCheckCaFpr.setPixmap(self.QUESTION_ICON)
 
@@ -569,7 +630,7 @@ class Wizard(QtGui.QWizard):
         Sets the status for the CA fingerprint check
         """
         self._complete_task(data, self.ui.lblCheckCaFpr)
-        passed = data[self._backend.PASSED_KEY]
+        passed = data[PASSED_KEY]
         if passed:
             self.ui.lblCheckApiCert.setPixmap(self.QUESTION_ICON)
 
@@ -613,10 +674,11 @@ class Wizard(QtGui.QWizard):
         Loads the services that the provider provides into the UI for
         the user to enable or disable.
         """
-        self.ui.grpServices.setTitle(
-            self.tr("Services by {0}").format(self._provider_details.name))
+        title = self.tr("Services by {0}").format(
+            self._provider_details['domain'])
+        self.ui.grpServices.setTitle(title)
 
-        services = get_supported(self._provider_details.services)
+        services = get_supported(self._provider_details['services'])
 
         for service in services:
             try:
@@ -658,32 +720,24 @@ class Wizard(QtGui.QWizard):
         if pageId == self.SETUP_PROVIDER_PAGE:
             if not self._provider_setup_ok:
                 self._reset_provider_setup()
-                sub_title = self.tr("Gathering configuration options for {0}")
-                sub_title = sub_title.format(self._provider_details.name)
-                self.page(pageId).setSubTitle(sub_title)
                 self.ui.lblDownloadCaCert.setPixmap(self.QUESTION_ICON)
-                self._provider_setup_defer = self._backend.\
-                    provider_bootstrap(self._domain)
+                self._backend.provider_bootstrap(provider=self._domain)
 
         if pageId == self.PRESENT_PROVIDER_PAGE:
-            sub_title = self.tr("Description of services offered by {0}")
-            sub_title = sub_title.format(self._provider_details.name)
-            self.page(pageId).setSubTitle(sub_title)
-
             details = self._provider_details
-            name = "<b>{0}</b>".format(details.name)
-            domain = "https://{0}".format(details.domain)
-            description = "<i>{0}</i>".format(details.description)
+            name = "<b>{0}</b>".format(details['name'])
+            domain = "https://{0}".format(details['domain'])
+            description = "<i>{0}</i>".format(details['description'])
             self.ui.lblProviderName.setText(name)
             self.ui.lblProviderURL.setText(domain)
             self.ui.lblProviderDesc.setText(description)
-            self.ui.lblServicesOffered.setText(details.services_string)
-            self.ui.lblProviderPolicy.setText(details.enrollment_policy)
+            self.ui.lblServicesOffered.setText(details['services_string'])
+            self.ui.lblProviderPolicy.setText(details['enrollment_policy'])
 
         if pageId == self.REGISTER_USER_PAGE:
-            sub_title = self.tr("Register a new user with {0}")
-            sub_title = sub_title.format(self._provider_details.name)
-            self.page(pageId).setSubTitle(sub_title)
+            title = self.tr("Register a new user with {0}")
+            title = title.format(self._provider_details['domain'])
+            self.page(pageId).setTitle(title)
             self.ui.chkRemember.setVisible(False)
 
         if pageId == self.SERVICES_PAGE:
@@ -726,37 +780,31 @@ class Wizard(QtGui.QWizard):
         """
         Connects all the backend signals with the wizard.
         """
-        sig = self._backend.signaler
-        sig.prov_name_resolution.connect(self._name_resolution)
-        sig.prov_https_connection.connect(self._https_connection)
-        sig.prov_download_provider_info.connect(self._download_provider_info)
-        sig.prov_get_details.connect(self._provider_get_details)
+        sig = self._leap_signaler
+        conntrack = self._connect_and_track
+        conntrack(sig.prov_name_resolution, self._name_resolution)
+        conntrack(sig.prov_https_connection, self._https_connection)
+        conntrack(sig.prov_download_provider_info,
+                  self._download_provider_info)
+        conntrack(sig.prov_get_details, self._provider_get_details)
+        conntrack(sig.prov_get_pinned_providers,
+                  self._load_configured_providers_with_pinned)
 
-        sig.prov_download_ca_cert.connect(self._download_ca_cert)
-        sig.prov_check_ca_fingerprint.connect(self._check_ca_fingerprint)
-        sig.prov_check_api_certificate.connect(self._check_api_certificate)
+        conntrack(sig.prov_download_ca_cert, self._download_ca_cert)
+        conntrack(sig.prov_check_ca_fingerprint, self._check_ca_fingerprint)
+        conntrack(sig.prov_check_api_certificate, self._check_api_certificate)
 
-        sig.srp_registration_finished.connect(self._registration_finished)
-        sig.srp_registration_failed.connect(self._registration_failed)
-        sig.srp_registration_taken.connect(self._registration_taken)
+        conntrack(sig.srp_registration_finished, self._registration_finished)
+        conntrack(sig.srp_registration_failed, self._registration_failed)
+        conntrack(sig.srp_registration_taken, self._registration_taken)
 
-    def _backend_disconnect(self):
+    def _disconnect_tracked(self):
         """
         This method is called when the wizard dialog is closed.
-        We disconnect all the backend signals in here.
+        We disconnect all the signals in here.
         """
-        sig = self._backend.signaler
-        try:
-            # disconnect backend signals
-            sig.prov_name_resolution.disconnect(self._name_resolution)
-            sig.prov_https_connection.disconnect(self._https_connection)
-            sig.prov_download_provider_info.disconnect(
-                self._download_provider_info)
-
-            sig.prov_download_ca_cert.disconnect(self._download_ca_cert)
-            sig.prov_check_ca_fingerprint.disconnect(
-                self._check_ca_fingerprint)
-            sig.prov_check_api_certificate.disconnect(
-                self._check_api_certificate)
-        except RuntimeError:
-            pass  # Signal was not connected
+        for signal, method in self._connected_signals:
+            try:
+                signal.disconnect(method)
+            except RuntimeError:
+                pass  # Signal was not connected
